@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, Net-ng.
+# Copyright (c) 2008, 2009 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -48,4 +48,4 @@ class Renderer(object):
         Prevent the common error of capturing the renderer into the closure
         of a callback
         """
-        raise Exception("A renderer object can't be used in a callback")
+        raise TypeError("can't pickle Renderer objects (are you using a renderer object in a callback ?)")
