@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, Net-ng.
+# Copyright (c) 2008, 2009 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -66,7 +66,7 @@ def head_render_css_url_test1():
     with h.head({'lang':'lang', 'dir':'dir', 'id':'id', 'profile':'profile'}):
         h << h.css_url('css')
 
-    assert h._css_url == {'/tmp/static_directory/css':0}
+    assert h._css_url == {'/tmp/static_directory/css' : (0, {})}
 
 
 def head_render_javascript_css_test2():
@@ -75,7 +75,7 @@ def head_render_javascript_css_test2():
     with h.head({'lang':'lang', 'dir':'dir', 'id':'id', 'profile':'profile'}):
         h << h.css_url('/css')
 
-    assert h._css_url == {'/css':0}
+    assert h._css_url == {'/css' : (0, {})}
 
 
 def head_render_css_url_test3():
@@ -85,8 +85,8 @@ def head_render_css_url_test3():
         h << h.css_url('/css')
         h << h.css_url('css')
 
-    assert h._css_url['/css'] == 0
-    assert h._css_url['/tmp/static_directory/css'] == 1
+    assert h._css_url['/css'] == (0, {})
+    assert h._css_url['/tmp/static_directory/css'] == (1, {})
 
 
 def head_render_javascript_url_test1():
@@ -95,7 +95,7 @@ def head_render_javascript_url_test1():
     with h.head({'lang':'lang', 'dir':'dir', 'id':'id', 'profile':'profile'}):
         h << h.javascript_url('test.js')
 
-    assert h._javascript_url == {'/tmp/static_directory/test.js':0}
+    assert h._javascript_url == {'/tmp/static_directory/test.js' : (0, {})}
 
 
 def head_render_javascript_url_test2():
@@ -103,7 +103,7 @@ def head_render_javascript_url_test2():
     h = xhtml.HeadRenderer('/tmp/static_directory/')
     with h.head({'lang':'lang', 'dir':'dir', 'id':'id', 'profile':'profile'}):
         h << h.javascript_url('/test.js')
-    assert h._javascript_url == {'/test.js':0}
+    assert h._javascript_url == {'/test.js' : (0, {})}
 
 
 def head_render_javascript_url_test3():
@@ -113,8 +113,8 @@ def head_render_javascript_url_test3():
         h << h.javascript_url('/test.js')
         h << h.javascript_url('test.js')
 
-    assert h._javascript_url['/test.js'] == 0
-    assert h._javascript_url['/tmp/static_directory/test.js'] == 1
+    assert h._javascript_url['/test.js'] == (0, {})
+    assert h._javascript_url['/tmp/static_directory/test.js'] == (1, {})
 
 
 def head_render_javascript_url_test4():
@@ -124,7 +124,7 @@ def head_render_javascript_url_test4():
         h << h.javascript_url('test.js')
         h << h.javascript_url('test.js')
 
-    assert h._javascript_url == {'/tmp/static_directory/test.js':0}
+    assert h._javascript_url == {'/tmp/static_directory/test.js' : (0, {})}
 
 
 def head_render_javascript_test1():
@@ -133,7 +133,7 @@ def head_render_javascript_test1():
     with h.head({'lang':'lang', 'dir':'dir', 'id':'id', 'profile':'profile'}):
         h << h.javascript('test', 'function test(arg1) { return true }')
 
-    assert h._javascript == {'test': (0, 'function test(arg1) { return true }')}
+    assert h._named_javascript == {'test': (0, 'function test(arg1) { return true }', {})}
 
 
 def head_render_javascript_test2():
@@ -146,8 +146,8 @@ def head_render_javascript_test2():
     with h.head({'lang':'lang', 'dir':'dir', 'id':'id', 'profile':'profile'}):
         h << h.javascript('test', js_method)
 
-    assert h._javascript_url == {'/static/nagare/pyjslib.js':0}
-    assert h._javascript.has_key('test')
+    assert h._javascript_url == {'/static/nagare/pyjslib.js' : (0, {})}
+    assert h._named_javascript.has_key('test')
 
 
 def head_render_javascript_test3():
@@ -157,7 +157,7 @@ def head_render_javascript_test3():
         h << h.javascript('test', 'function test(arg1) { return true }')
         h << h.javascript('test', 'function test(arg1) { return true }')
 
-    assert h._javascript == {'test': (0, 'function test(arg1) { return true }')}
+    assert h._named_javascript == {'test' : (0, 'function test(arg1) { return true }', {})}
 
 
 def head_render_render_test1():
@@ -298,7 +298,7 @@ def html_render_parse_html_test5():
     """ XHTML namespace unit test - HTMLRender - parse_html - get html from an URL """
     try:
         h = xhtml.Renderer()
-        root = h.parse_html('http://www.google.fr')
+        root = h.parse_html('http://www.google.fr/')
     except:
         assert False
     else:
@@ -518,7 +518,7 @@ def html_render_select_test2():
     options = h.root.xpath('.//option')
     for option in options:
         option.selected('option2')
-        
+
     assert options[1].attrib['selected'] == 'selected'
 
 
@@ -867,7 +867,7 @@ def html_render_img_test1():
 
 def html_render_img_test2():
     """ XHTML namespace unit test - Tag img - init - relative source """
-    h = xhtml.Renderer(xhtml.DummyRenderer(static='/tmp/static/'))
+    h = xhtml.Renderer(static='/tmp/static/')
 
     with h.html:
         with h.body:
@@ -878,7 +878,7 @@ def html_render_img_test2():
 
 def html_render_img_test3():
     """ XHTML namespace unit test - Tag img - init - absolute source """
-    h = xhtml.Renderer(xhtml.DummyRenderer(static='/tmp/static/'))
+    h = xhtml.Renderer(static_url='/tmp/static/')
 
     with h.html:
         with h.body:
@@ -913,7 +913,7 @@ def html_render_a_test1():
 
 def html_render_img_test2():
     """ XHTML namespace unit test - Tag a - init - relative source """
-    h = xhtml.Renderer(xhtml.DummyRenderer(static='/tmp/static/'))
+    h = xhtml.Renderer(static_url='/tmp/static/')
 
     with h.html:
         with h.body:
@@ -924,7 +924,7 @@ def html_render_img_test2():
 
 def html_render_action_test1():
     """ XHTML namespace unit test - action - put action method on tag a """
-    h = xhtml.Renderer(xhtml.DummyRenderer(callbacks=callbacks.Callbacks(), static='/tmp/static/'))
+    h = xhtml.Renderer(callbacks=callbacks.Callbacks(), static_url='/tmp/static/')
 
     with h.html:
         with h.body:
@@ -936,19 +936,19 @@ def html_render_action_test1():
 
 def html_render_action_test2():
     """ XHTML namespace unit test - action - put action method on tag a (replace existing href) """
-    h = xhtml.Renderer(xhtml.DummyRenderer(callbacks=callbacks.Callbacks(), static='/tmp/static/'))
+    h = xhtml.Renderer(callbacks=callbacks.Callbacks(), static_url='/tmp/static/')
 
     with h.html:
         with h.body:
             a = h.a(href='http://www.google.com').action(lambda x: None)
             h << a
 
-    assert a.attrib['href'] != 'http://www.google.com'
+    assert a.attrib['href'] == 'http://www.google.com'
 
 
 def html_render_action_test3():
     """ XHTML namespace unit test - action - put action method on tag imagge """
-    h = xhtml.Renderer(xhtml.DummyRenderer(callbacks=callbacks.Callbacks(), static='/tmp/static/'))
+    h = xhtml.Renderer(callbacks=callbacks.Callbacks(), static_url='/tmp/static/')
 
     with h.html:
         with h.body:
@@ -960,7 +960,7 @@ def html_render_action_test3():
 
 def html_render_action_test4():
     """ XHTML namespace unit test - action - put action method on tag a (replace existing href) """
-    h = xhtml.Renderer(xhtml.DummyRenderer(callbacks=callbacks.Callbacks(), static='/tmp/static/'))
+    h = xhtml.Renderer(callbacks=callbacks.Callbacks(), static_url='/tmp/static/')
 
     with h.html:
         with h.body:
@@ -972,7 +972,7 @@ def html_render_action_test4():
 
 def html_render_action_test5():
     """ XHTML namespace unit test - asynchronous render - action - put action method on tag a """
-    h = xhtml.AsyncRenderer(xhtml.DummyRenderer(callbacks=callbacks.Callbacks(), static='/tmp/static/'))
+    h = xhtml.AsyncRenderer(callbacks=callbacks.Callbacks(), static_url='/tmp/static/')
 
     with h.html:
         with h.body:
@@ -984,7 +984,7 @@ def html_render_action_test5():
 
 def html_render_action_test6():
     """ XHTML namespace unit test - asynchronous render - action - put action method on tag a (replace existing href) """
-    h = xhtml.AsyncRenderer(xhtml.DummyRenderer(callbacks=callbacks.Callbacks(), static='/tmp/static/'))
+    h = xhtml.AsyncRenderer(callbacks=callbacks.Callbacks(), static_url='/tmp/static/')
 
     with h.html:
         with h.body:
@@ -996,7 +996,7 @@ def html_render_action_test6():
 
 def html_render_action_test7():
     """ XHTML namespace unit test - asynchronous render - action - put action method on tag imagge """
-    h = xhtml.AsyncRenderer(xhtml.DummyRenderer(callbacks=callbacks.Callbacks(), static='/tmp/static/'))
+    h = xhtml.AsyncRenderer(callbacks=callbacks.Callbacks(), static_url='/tmp/static/')
 
     with h.html:
         with h.body:
@@ -1008,7 +1008,7 @@ def html_render_action_test7():
 
 def html_render_action_test8():
     """ XHTML namespace unit test - asynchronous render - action - put action method on tag a (replace existing href) """
-    h = xhtml.AsyncRenderer(xhtml.DummyRenderer(callbacks=callbacks.Callbacks(), static='/tmp/static/'))
+    h = xhtml.AsyncRenderer(callbacks=callbacks.Callbacks(), static_url='/tmp/static/')
 
     with h.html:
         with h.body:
