@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, Net-ng.
+# Copyright (c) 2008, 2009 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -9,15 +9,16 @@
 
 """The XHTML renderer
 
-This renderer is dedicated to the framework
+This renderer is dedicated to the Nagare framework
 """
 
 from __future__ import with_statement
 
-import types, urllib, imghdr
+import operator, types, urllib, imghdr
 
 from lxml import etree as ET
 import peak.rules
+import webob
 
 from nagare import security, ajax, presentation
 
@@ -30,8 +31,8 @@ from nagare.namespaces import xhtml_base
 # Generic methods to add an attribute to a tag
 # --------------------------------------------
 
-@peak.rules.when(xml.add_attribut, (xml._Tag, basestring, ajax.Update))
-def add_attribut(self, name, value):
+@peak.rules.when(xml.add_attribute, (xml._Tag, basestring, ajax.Update))
+def add_attribute(self, name, value):
     """Add an attribute with a ``ajax.Update`` value
     
     In:
@@ -40,11 +41,11 @@ def add_attribut(self, name, value):
       - ``value`` -- ``ajax.Update`` value
     """
     # Generate a XHR request
-    add_attribut(self, name, value.generate_action(3, self.renderer))
+    xml.add_attribute(self, name, value.generate_action(self._actions[0], self.renderer))
 
-@peak.rules.when(xml.add_attribut, (xml._Tag, basestring, types.FunctionType))
-def add_attribut(self, name, value):
-    """Add an attribut with a function value
+@peak.rules.when(xml.add_attribute, (xml._Tag, basestring, types.FunctionType))
+def add_attribute(self, name, value):
+    """Add an attribute with a function value
     
     In:
       - ``self`` -- the tag
@@ -52,11 +53,11 @@ def add_attribut(self, name, value):
       - ``value`` -- function value
     """
     # Transcode the function to javascript
-    add_attribut(self, name, ajax.JS(value))
+    xml.add_attribute(self, name, ajax.JS(value))
 
-@peak.rules.when(xml.add_attribut, (xml._Tag, basestring, types.MethodType))
-def add_attribut(self, name, value):
-    """Add an attribut with a method value
+@peak.rules.when(xml.add_attribute, (xml._Tag, basestring, types.MethodType))
+def add_attribute(self, name, value):
+    """Add an attribute with a method value
     
     In:
       - ``self`` -- the tag
@@ -64,11 +65,11 @@ def add_attribut(self, name, value):
       - ``value`` -- method value
     """
     # Transcode the method to javascript
-    add_attribut(self, name, ajax.JS(value))
+    xml.add_attribute(self, name, ajax.JS(value))
 
-@peak.rules.when(xml.add_attribut, (xml._Tag, basestring, ajax.JS))
-def add_attribut(self, name, value):
-    """Add an attribut with a ``ajax.JS`` value
+@peak.rules.when(xml.add_attribute, (xml._Tag, basestring, ajax.JS))
+def add_attribute(self, name, value):
+    """Add an attribute with a ``ajax.JS`` value
     
     In:
       - ``self`` -- the tag
@@ -98,48 +99,67 @@ def absolute_url(url, static):
             url = static + url
         
     return url
-        
+
 
 class HeadRenderer(xhtml_base.HeadRenderer):
     """The XHTML head Renderer
     
     This renderer knows about the static contents of the application
-    """    
-    def __init__(self, static):
+    """
+    def __init__(self, static_url):
+        """Renderer initialisation
+        
+        The ``HeadRenderer`` keeps track of the javascript and css used by every views,
+        to be able to concatenate them into the ``<head>`` section.
+        """
         super(HeadRenderer, self).__init__()
 
         # Directory where are located the static contents of the application
-        self.static = static
-        self.anonymous = 0
-
-    def add_script(self, script, url):
-        if url:
-            self.javascript_url(url)
-        else:
-            self.javascript(self.anonymous, script)
-            self.anonymous += 1
+        self.static_url = static_url
         
-    def add_css(self, css):
-        self.css(self.anonymous, css)
-        self.anonymous += 1
+        self._named_css = {}        # CSS code
+        self._css_url = {}          # CSS URLs
+        self._named_javascript = {} # Javascript code
+        self._javascript_url = {}   # Javascript URLs
 
-    def css_url(self, url):
-        """Memorize a css style URL
+        self._order = 0             # Memorize the order of the javascript and css
+
+    def css(self, name, style, **kw):
+        """Memorize an in-line named css style
         
         In:
-          - ``url`` -- the css style URL
+          - ``name`` -- unique name of this css style (to prevent double definition)
+          - ``style`` -- the css style
+          - ``kw`` -- attributes of the generated ``<style>`` tag
           
         Return:
           - ``()``
         """
-        return super(HeadRenderer, self).css_url(absolute_url(url, self.static))
+        self._named_css.setdefault(name, (self._order, style, kw))
+        self._order += 1
+        return ()
 
-    def javascript(self, name, script):
-        """Memorize a in-line javascript code
+    def css_url(self, url, **kw):
+        """Memorize a css style URL
+        
+        In:
+          - ``url`` -- the css style URL
+          - ``kw`` -- attributes of the generated ``<link>`` tag
+          
+        Return:
+          - ``()``
+        """
+        self._css_url.setdefault(absolute_url(url, self.static_url), (self._order, kw))
+        self._order += 1
+        return ()
+
+    def javascript(self, name, script, **kw):
+        """Memorize an in-line named javascript code
         
         In:
           - ``name`` -- unique name of this javascript code (to prevent double definition)
-          - ``script`` -- a function, a method or a javascript code
+          - ``script`` -- the javascript code
+          - ``kw`` -- attributes of the generated ``<script>`` tag
           
         Return:
           - ``()``
@@ -153,18 +173,62 @@ class HeadRenderer(xhtml_base.HeadRenderer):
             self.javascript_url('/static/nagare/pyjslib.js')
             script = script.javascript
 
-        return super(HeadRenderer, self).javascript(name, script)
+        self._named_javascript.setdefault(name, (self._order, script, kw))
+        self._order += 1
+        return ()
 
-    def javascript_url(self, url):
+    def javascript_url(self, url, **kw):
         """Memorize a javascript URL
         
         In:
           - ``url`` -- the javascript URL
+          - ``kw`` -- attributes of the the generated ``<script>`` tag
           
         Return:
           - ``()``
         """
-        return super(HeadRenderer, self).javascript_url(absolute_url(url, self.static))
+        self._javascript_url.setdefault(absolute_url(url, self.static_url), (self._order, kw))
+        self._order += 1
+        return ()
+
+    def _style(self, append, tag, style):
+        append(tag, style)
+    
+    def _script(self, append, tag, script):
+        append(tag, script)
+
+    def _get_named_css(self):
+        """Return the list of the in-line named css styles, sorted by order of insertion
+        
+        Return:
+          - list of (name, css style, attributes)
+        """
+        
+        return [(name, style, attributes) for (name, (order, style, attributes)) in sorted(self._named_css.items(), key=operator.itemgetter(1))]
+
+    def _get_css_url(self):
+        """Return the list of css URLs, sorted by order of insertion
+        
+        Return:
+          - list of css (URLs, attributes)
+        """
+        return [(url, attributes) for (url, (order, attributes)) in sorted(self._css_url.items(), key=operator.itemgetter(1))]
+
+    def _get_named_javascript(self):
+        """Return the list of named javascript codes, sorted by order of insertion
+        
+        Return:
+          - list of (name, javascript code, attributes)
+        """
+        return [(name, js, attributes) for (name, (order, js, attributes)) in sorted(self._named_javascript.items(), key=operator.itemgetter(1))]
+
+    def _get_javascript_url(self):
+        """Return the list of javascript URLs, sorted by order of insertion
+        
+        Return:
+          - list of javascript (URLs, attributes)
+        """
+        return [(url, attributes) for (url, (order, attributes)) in sorted(self._javascript_url.items(), key=operator.itemgetter(1))]
 
 @presentation.render_for(HeadRenderer)
 def render(self, h, *args):
@@ -180,15 +244,6 @@ def render(self, h, *args):
     
     # Create the tags to include the CSS styles and the javascript codes
 
-    css_url = [self.link(rel='stylesheet', type='text/css', href='%s' % url) for url in self._get_css_url()]
-    
-    css = self._get_css()
-    if css:
-        css = self.style(css, type='text/css')
-
-    javascript_url = [self.script(src=url, type='text/javascript') for url in self._get_javascript_url()]
-    javascript = [self.script(script.encode('utf-8'), type='text/javascript') for script in self._get_javascript()]
-
     head = self.root
 
     if isinstance(head, ET.ElementBase) and (head.tag == 'head'):
@@ -196,9 +251,14 @@ def render(self, h, *args):
         head = self.head(head[:], dict(head.attrib))
     else:
         head = self.head(head)
-    
-    return head(css_url, css, javascript_url, javascript, '\n')
 
+    head.extend([self.link(rel='stylesheet', type='text/css', href=url, **attributes) for (url, attributes) in self._get_css_url()])
+    head.extend([self.script(type='text/javascript', src=url, **attributes) for (url, attributes) in self._get_javascript_url()])
+
+    head.extend([self.style(css, type='text/css', **attributes) for (name, css, attributes) in self._get_named_css()])
+    head.extend([self.script(js, type='text/javascript', **attributes) for (name, js, attributes) in self._get_named_javascript()])
+
+    return head
 
 # ----------------------------------------------------------------------------------
 
@@ -240,14 +300,14 @@ class _HTMLActionTag(xhtml_base._HTMLTag):
     async_action = sync_action
     
     def _async_action(self, renderer, action, permissions, subject):
-        """Register a asynchronous action
+        """Register an asynchronous action
         
         In:
           - ``renderer`` -- the current renderer
           - ``action`` -- the action
         """
         if callable(action):
-            action = ajax.Update(renderer.model, action, None)
+            action = ajax.Update(action=action)
         
         self.set(self._actions[2], action.generate_action(self._actions[0], renderer))
 
@@ -310,7 +370,7 @@ class Form(xhtml_base._HTMLTag):
             action = security.permissions_with_subject(permissions, subject or self._renderer.component())(action)
 
         # Generate a hidden field with the action attached
-        self.append(self.renderer.input(type='hidden', name=self.renderer.register_callback(0, action)))
+        self.append(self.renderer.div(self.renderer.input(type='hidden', name=self.renderer.register_callback(0, action))))
         return self
         
     def post_action(self, action, permissions=None, subject=None):
@@ -330,7 +390,7 @@ class Form(xhtml_base._HTMLTag):
              action = security.permissions_with_subject(permissions, subject or self._renderer.component())(action)
 
         # Generate a hidden field with the action attached
-        self.append(self.renderer.input(type='hidden', name=self.renderer.register_callback(3, action)))
+        self.append(self.renderer.div(self.renderer.input(type='hidden', name=self.renderer.register_callback(3, action))))
         return self
     
 # ----------------------------------------------------------------------------------
@@ -341,8 +401,8 @@ class TextInput(_HTMLActionTag):
 
     # Tuple:
     #   - action type
-    #   - name of the attribute for the synchronous actions
-    #   - name of the attribute for then asynchronous actions
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
     _actions = (1, 'name', 'onchange')
 
     def __call__(self, *children, **attrib):
@@ -364,8 +424,8 @@ class TextArea(_HTMLActionTag):
     """
     # Tuple:
     #   - action type
-    #   - name of the attribute for the synchronous actions
-    #   - name of the attribute for then asynchronous actions
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
     _actions = (1, 'name', 'onchange')
     
     def action(self, action, permissions=None, subject=None):
@@ -389,8 +449,8 @@ class PasswordInput(_HTMLActionTag):
 
     # Tuple:
     #   - action type
-    #   - name of the attribute for the synchronous actions
-    #   - name of the attribute for then asynchronous actions
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
     _actions = (1, 'name', 'onchange')
 
 
@@ -400,9 +460,9 @@ class RadioInput(_HTMLActionTag):
 
     # Tuple:
     #   - action type
-    #   - name of the attribute for the synchronous actions
-    #   - name of the attribute for then asynchronous actions
-    _actions = (2, 'value', 'onchange')
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
+    _actions = (2, 'value', 'onclick')
 
     def selected(self, flag):
         """(de)Select the tag
@@ -427,9 +487,9 @@ class CheckboxInput(_HTMLActionTag):
 
     # Tuple:
     #   - action type
-    #   - name of the attribute for the synchronous actions
-    #   - name of the attribute for then asynchronous actions
-    _actions = (1, 'name', 'onchange')
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
+    _actions = (1, 'name', 'onclick')
 
     def selected(self, flag):
         """(de)Select the tag
@@ -454,8 +514,8 @@ class SubmitInput(_HTMLActionTag):
 
     # Tuple:
     #   - action type
-    #   - name of the attribute for the synchronous actions
-    #   - name of the attribute for then asynchronous actions
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
     _actions = (4, 'name', 'onclick')
     
     def async_action(self, renderer, action, permissions, subject):
@@ -467,8 +527,8 @@ class HiddenInput(_HTMLActionTag):
 
     # Tuple:
     #   - action type
-    #   - name of the attribute for the synchronous actions
-    #   - name of the attribute for then asynchronous actions
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
     _actions = (1, 'name', 'onchange')
 
 
@@ -478,8 +538,8 @@ class FileInput(_HTMLActionTag):
 
     # Tuple:
     #   - action type
-    #   - name of the attribute for the synchronous actions
-    #   - name of the attribute for then asynchronous actions
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
     _actions = (1, 'name', 'onchange')
 
     def init(self, renderer):
@@ -503,8 +563,8 @@ class ImageInput(_HTMLActionTag):
 
     # Tuple:
     #   - action type
-    #   - name of the attribute for the synchronous actions
-    #   - name of the attribute for then asynchronous actions
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
     _actions = (5, 'name', 'onclick')
 
 # ----------------------------------------------------------------------------------
@@ -539,8 +599,8 @@ class Select(_HTMLActionTag):
 
     # Tuple:
     #   - action type
-    #   - name of the attribute for the synchronous actions
-    #   - name of the attribute for then asynchronous actions
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
     _actions = (1, 'name', 'onchange')
     
     def action(self, action, permissions=None, subject=None):
@@ -566,6 +626,13 @@ class Select(_HTMLActionTag):
 class A(_HTMLActionTag):
     """ ``<a>`` tags
     """
+
+    # Tuple:
+    #   - action type
+    #   - name of the attribute for the synchronous action
+    #   - name of the attribute for the asynchronous action
+    _actions = (41, None, 'onclick')
+
     def sync_action(self, renderer, action, permissions, subject):
         """Register a synchronous action
         
@@ -580,7 +647,8 @@ class A(_HTMLActionTag):
             # Wrapper the ``action`` into a wrapper that will check the user permissions
             action = security.permissions_with_subject(permissions, subject or self._renderer.component())(action)
 
-        self.set('href', renderer.add_sessionid_in_url( self.get('href', ''), (renderer.register_callback(4, action),)))
+        href = self.get('href', '').partition('#')
+        self.set('href', renderer.add_sessionid_in_url(href[0], (renderer.register_callback(4, action),))+href[1]+href[2])
     
     def _async_action(self, renderer, action, permissions, subject):
         """Register an asynchronous action
@@ -591,11 +659,8 @@ class A(_HTMLActionTag):
           - ``permissions`` -- permissions needed to execute the action
           - ``subject`` -- subject to test the permissions on
         """
-        if callable(action):
-            action = ajax.Update(renderer.model, action, None)
-        
+        super(A, self)._async_action(renderer, action, permissions, subject)
         self.set('href', '#')
-        self.set('onclick', action.generate_action(41, renderer))
     async_action = _async_action
     
     """
@@ -612,8 +677,8 @@ class A(_HTMLActionTag):
         return self
     """
 
-@peak.rules.when(xml.add_attribut, (A, basestring, basestring))
-def add_attribut(next_method, self, name, value):
+@peak.rules.when(xml.add_attribute, (A, basestring, basestring))
+def add_attribute(next_method, self, name, value):
     if name == 'href':
         value = absolute_url(value, self._renderer.url)
 
@@ -623,25 +688,29 @@ def add_attribut(next_method, self, name, value):
 
 class Img(_HTMLActionTag):
     """ ``<img>`` tags
-    """    
-    def _set_content_type(self, response, img):
-        """Guess the image format and set the ``Content-Type`` of the response
+    """
+    def _set_content_type(self, action):
+        """Generate the image and guess its format
         
         In:
-          - ``reponse`` -- the response object
-          - ``img`` -- the image data
+          - ``action`` -- function to call to generate the image data
           
         Return:
-          - ``img``
+          - new response object raised
         """
-        if not response.content_type:
+        e = webob.exc.HTTPOk(headerlist=[('Content-Type', '')])
+        img = action(e)
+        e.body = img
+        
+        content_type = e.content_type
+        if not content_type:
             # If no ``Content-Type`` is already set, use the ``imghdr`` module
             # to guess the format of the image
-            img_type = imghdr.what(None, img[:32])
-            if img_type is not None:
-                response.content_type = 'image/'+img_type
-        return img
+            content_type = 'image/'+(imghdr.what(None, img[:32]) or '*')
+        e.content_type = content_type
         
+        raise e
+    
     def sync_action(self, renderer, action, permissions, subject):
         """Register a synchronous action
         
@@ -651,9 +720,9 @@ class Img(_HTMLActionTag):
           - ``renderer`` -- the current renderer
           - ``action`` -- the action
         """        
-        self.set('src', renderer.add_sessionid_in_url(sep=';') + ';' + renderer.register_callback(6, None, lambda h: self._set_content_type(h.response, action(h))))
+        self.set('src', renderer.add_sessionid_in_url(sep=';') + ';' + renderer.register_callback(2, lambda: self._set_content_type(action)))
     async_action = sync_action
-  
+
     def add_child(self, child):
         """Add attributes to the image
         
@@ -666,7 +735,7 @@ class Img(_HTMLActionTag):
         # of the application
         src = self.get('src')
         if src is not None:
-            self.set('src', absolute_url(src, self.renderer.head.static))
+            self.set('src', absolute_url(src, self.renderer.head.static_url))
 
 
 # ----------------------------------------------------------------------------------
@@ -691,37 +760,44 @@ class Label(xhtml_base._HTMLTag):
 
 # ----------------------------------------------------------------------------------
 
-class DummyRenderer(xhtml_base.DummyRenderer):
-    """Root of the ``xhtml.Renderer`` objects"""
-    head_renderer_factory = HeadRenderer
-        
-    def __init__(self, session=None, request=None, response=None, callbacks=None, static='', url='/'):
-        """Initialisation
-        
-        In:
-          - ``session`` -- the session object
-          - ``request`` -- the request object
-          - ``response`` -- the response object
-          - ``callbacks`` -- the registered actions on the tags
-          - ``static`` -- directory of the static contents of the application
-          - ``url`` -- url prefix of the application
-        """
-        super(DummyRenderer, self).__init__(static)
+class Script(xhtml_base._HTMLTag):
+    pass
 
-        self.session = session
-        self.request = request
-        self.response = response        
-        self._callbacks = callbacks
-        self._rendered = set()      # List a the component rendered by all the renderers
-        self.url = url
-        
-        self.component = None        
-        #self.model = None
+@peak.rules.when(xml.add_child, (xhtml_base._HTMLTag, Script))
+def add_child(next_method, self, script):
+    """Add a <script> to a tag
+    
+    In:
+      - ``self`` -- the tag
+      - ``script`` -- the script to add
+    """
+    self.renderer.head._script(next_method, self, script)
+    return ''
+
+
+class Style(xhtml_base._HTMLTag):
+    pass
+
+@peak.rules.when(xml.add_child, (xhtml_base._HTMLTag, Style))
+def add_child(next_method, self, style):
+    """Add a <style> to a tag
+    
+    In:
+      - ``self`` -- the tag
+      - ``style`` -- the style to add
+    """
+    self.renderer.head._style(next_method, self, style)
+    return ''
+
+
+# ----------------------------------------------------------------------------------
 
 class Renderer(xhtml_base.Renderer):
     """The XHTML synchronous renderer
     """
     
+    head_renderer_factory = HeadRenderer
+
     # Redefinition of the he HTML tags with actions
     # ---------------------------------------------
 
@@ -738,6 +814,9 @@ class Renderer(xhtml_base.Renderer):
     select = TagProp('select', set(xhtml_base.allattrs + ('name', 'size', 'multiple', 'disabled', 'tabindex', 'onfocus', 'onblur', 'onchange', 'rows')), Select)
     textarea = TagProp('textarea', set(xhtml_base.allattrs + xhtml_base.focusattrs + ('name', 'rows', 'cols', 'disabled', 'readonly', 'onselect', 'onchange', 'wrap')), TextArea)
 
+    script = TagProp('script', set(('id', 'charset', 'type', 'language', 'src', 'defer')), Script)
+    style = TagProp('style', set(xhtml_base.i18nattrs+('id', 'type', 'media', 'title')), Style)
+    
     _specialTags = dict(
                     text_input     = TextInput,
                     radio_input    = RadioInput,
@@ -769,6 +848,41 @@ class Renderer(xhtml_base.Renderer):
         cls._custom_lookup = CustomLookup(cls._specialTags, ET.ElementDefaultClassLookup(element=xhtml_base._HTMLTag))
         cls._html_parser = ET.HTMLParser()
         cls._html_parser.setElementClassLookup(cls._custom_lookup)
+        
+    def __init__(self, parent=None, session=None, request=None, response=None, callbacks=None, static_url='', static_path='', url='/'):
+        """Renderer initialisation
+        
+        In:
+          - ``parent`` -- parent renderer
+          - ``session`` -- the session object
+          - ``request`` -- the request object
+          - ``response`` -- the response object
+          - ``callbacks`` -- the registered actions on the tags
+          - ``static_url`` -- url of the static contents of the application
+          - ``static_path`` -- path of the static contents of the application
+          - ``url`` -- url prefix of the application
+        """
+        super(Renderer, self).__init__(parent, static_url=static_url)
+
+        if parent is None:
+            self.session = session
+            self.request = request
+            self.response = response
+            self._callbacks = callbacks
+            self.static_path = static_path
+            self.url = url
+            self._rendered = set()
+        else:
+            self.session = parent.session
+            self.request = parent.request
+            self.response = parent.response
+            self._callbacks = parent._callbacks
+            self.static_path = parent.static_path
+            self.url = parent.url
+            self._rendered = parent._rendered
+        
+        self.component = None
+        self.model = None
 
     def makeelement(self, tag):
         """Make a tag
@@ -781,6 +895,26 @@ class Renderer(xhtml_base.Renderer):
         """                
         return self._makeelement(tag, self._html_parser)
 
+    def parse_xml(self, source, fragment=False, no_leading_text=False, **kw):
+        """Parse a XML file
+        
+        In:
+          - ``source`` -- can be a filename or a file object
+          - ``fragment`` -- if ``True``, can parse a XML fragment i.e a XML without
+            a unique root
+          - ``no_leading_text`` -- if ``fragment`` is ``True``, ``no_leading_text``
+            is ``False`` and the XML to parsed begins by a text, this text is keeped
+          - ``kw`` -- keywords parameters are passed to the XML parser
+          
+        Return:
+          - the root element of the parsed XML, if ``fragment`` is ``False``
+          - a list of XML elements, if ``fragment`` is ``True`` 
+        """
+        if isinstance(source, basestring):
+            source = absolute_url(source, self.static_path)
+
+        return super(Renderer, self).parse_xml(source, fragment, no_leading_text, **kw)
+
     def parse_html(self, source, fragment=False, no_leading_text=False, xhtml=False, **kw):
         """Parse a (X)HTML file
         
@@ -797,32 +931,14 @@ class Renderer(xhtml_base.Renderer):
           - the root element of the parsed HTML, if ``fragment`` is ``False``
           - a list of HTML elements, if ``fragment`` is ``True`` 
         """
+        if isinstance(source, basestring):
+            source = absolute_url(source, self.static_path)
+
         parser = ET.XMLParser(**kw) if xhtml else ET.HTMLParser(**kw)
         parser.setElementClassLookup(self._custom_lookup)
         
         return self._parse_html(parser, source, fragment, no_leading_text, **kw)
 
-    def __init__(self, parent=None):
-        """Renderer initialisation
-        
-        In:
-          - ``parent`` -- parent renderer
-        """        
-        if parent is None:
-            parent = DummyRenderer()
-
-        super(Renderer, self).__init__(parent)
-
-        self.session = parent.session
-        self.request = parent.request
-        self.response = parent.response
-        self._callbacks = parent._callbacks
-        self._rendered = parent._rendered
-        self.component = parent.component
-        self.url = parent.url
-        self.head = parent.head
-        
-        self.model = None
 
     def start_rendering(self, component, model):
         """Method called before to render a component
@@ -837,11 +953,11 @@ class Renderer(xhtml_base.Renderer):
         if component.url is not None:
             self.url = self.url + '/' + component.url
 
-        # Delete all the previous callbacks registered by the component
+        # Delete all the previous callbacks registered by this component
         if self._callbacks and (component not in self._rendered):
             self._callbacks.unregister_callbacks(component)
         
-        # Memorize all the component rendered
+        # Memorize all the rendered components
         self._rendered.add(component)
 
     def action(self, tag, action, permissions, subject):
@@ -896,7 +1012,7 @@ class Renderer(xhtml_base.Renderer):
           - ``form`` -- the form tag
         """
         if self.session:
-            form(self.session.sessionid_in_form(self, self.request, self.response))
+            form(self.div(self.session.sessionid_in_form(self, self.request, self.response)))
 
     def add_sessionid_in_url(self, u='', params=None, sep='&'):
         """Add the session and continuation ids into an url
@@ -910,7 +1026,8 @@ class Renderer(xhtml_base.Renderer):
         Return:
           - the completed url
         """
-        if not u.startswith('/'):
+        i = u.find(':')
+        if ((i == -1) or not u[:i].isalpha()) and u and (u[0] != '/'):
             u = self.url + '/' + u
         
         if params is None:
@@ -920,9 +1037,68 @@ class Renderer(xhtml_base.Renderer):
             u += '?' + sep.join(self.session.sessionid_in_url(self.request, self.response) + params)
 
         return u
-    
 
-@presentation.render_for(HeadRenderer, model='async')
+
+class AsyncHeadRenderer(HeadRenderer):
+    def __init__(self, static_url):
+        """Renderer initialisation
+        
+        The ``HeadRenderer`` keeps track of the javascript and css used by every views,
+        to be able to concatenate them into the ``<head>`` section.
+        """
+        super(AsyncHeadRenderer, self).__init__(static_url=static_url)
+                
+        self._anonymous_css = []         # CSS
+        self._anonymous_javascript = []  # Javascript code
+
+    def _css(self, style):
+        """Memorize an in-line anonymous css style
+        
+        In:
+          - ``style`` -- the css style
+        """
+        self._anonymous_css.append((self._order, style))
+        self._order += 1
+
+    def _javascript(self, script):
+        """Memorize an in-line anonymous javascript code
+        
+        In:
+          - ``script`` -- the javascript code
+        """
+        self._anonymous_javascript.append((self._order, script))
+        self._order += 1
+
+    def _style(self, append, tag, style):
+        self._css(style.text or '')
+    
+    def _script(self, append, tag, script):    
+        url = script.get('src')
+        
+        if url:
+            self.javascript_url(url)
+        else:
+            self._javascript(script.text or '')
+
+    def _get_anonymous_css(self):
+        """Return the list of the in-line anonymous css styles, sorted by order of insertion
+        
+        Return:
+          - list of css styles
+        """
+        return [css for (order, css) in sorted(self._anonymous_css)]
+        
+
+    def _get_anonymous_javascript(self):
+        """Return the list of anonymous javascript codes, sorted by order of insertion
+        
+        Return:
+          - list of javascript codes
+        """
+        return [js for (order, js) in sorted(self._anonymous_javascript)]
+
+    
+@presentation.render_for(AsyncHeadRenderer)
 def render(self, h, *args):
     """Generate a javascript view of the head
     
@@ -930,71 +1106,43 @@ def render(self, h, *args):
       - ``h`` -- the current renderer
       
     Return:
-      - a javascript
+      - a javascript string
     """
-    css = ' '.join(self._get_css())         # Inline CSS    
-    js = ';'.join(self._get_javascript())   # Javascript codes
-
-    return "nagare_loadAll(%s, %s, '%s', '%s')" % (
-                                                 ajax.py2js(self._get_javascript_url(), h),
-                                                 ajax.py2js(self._get_css_url(), h),
-                                                 css.replace("'", r"\'").replace('\n', ''),
-                                                 js.replace("'", r"\'").replace('\n', '').encode('utf-8')
-                                                )
-
-
-class AsyncScript(xhtml_base._HTMLTag):
-    pass
-
-
-@peak.rules.when(xml.add_child, (xhtml_base._HTMLTag, AsyncScript))
-def add_child(self, script):
-    """Add a script to a tag
-    
-    In:
-      - ``self`` -- the tag
-      - ``script`` -- the script to add
-    """
-    # Don't generate any tags but remember the script text or url
-    self.renderer.head.add_script(script.text, script.get('src'))
-    return ''
-
-
-class AsyncStyle(xhtml_base._HTMLTag):
-    pass
-
-
-@peak.rules.when(xml.add_child, (xhtml_base._HTMLTag, AsyncStyle))
-def add_child(self, css):
-    """Add a css to a tag
-    
-    In:
-      - ``self`` -- the tag
-      - ``css`` -- the script to add
-    """
-    # Don't generate any tags but remember the css text
-    self.renderer.head.add_css(css.text)
-    return ''
+    return "nagare_loadAll(%s, %s, %s, %s, %s, %s)" % (
+                                                         ajax.py2js(self._get_named_css(), h),
+                                                         ajax.py2js(r'\n'.join(self._get_anonymous_css()), h),
+                                                         ajax.py2js(self._get_css_url(), h),
+                                                         ajax.py2js(self._get_named_javascript(), h),
+                                                         ajax.py2js(';'.join(self._get_anonymous_javascript()), h),
+                                                         ajax.py2js(self._get_javascript_url(), h)
+                                                      )
 
 
 class AsyncRenderer(Renderer):
     """The XHTML asynchronous renderer
     """
-    script = TagProp('script', set(('id', 'charset', 'type', 'language', 'src', 'defer')), AsyncScript)
-    style = TagProp('style', set(Renderer.i18nattrs+('id', 'type', 'media', 'title')), AsyncStyle)
-        
-    _specialTags = Renderer._specialTags.copy()
+    head_renderer_factory = AsyncHeadRenderer
 
-    def __init__(self, parent):
+    def __init__(self, *args, **kw):
         """Renderer initialisation
         
         In:
           - ``parent`` -- parent renderer
-        """        
-        super(AsyncRenderer, self).__init__(parent)
+        """
+        super(AsyncRenderer, self).__init__(*args, **kw)
+
         self.async_root = True;
         self.wrapper_to_generate = False    # Add a ``<div>`` around the rendering ?
+
+    def javascript_url(self, url):
+        self.head.javascript_url(url)
         
+    def _javascript(self, js):
+        self.head._javascript(js)
+        
+    def _css(self, style):
+        self.head._css(style)
+
     def action(self, tag, action, permissions, subject):
         """Register an asynchronous action on a tag
         
@@ -1069,4 +1217,4 @@ if __name__ == '__main__':
                         with h.td:
                             h << column
 
-    print h.html(h.head.head(h.head.render()), h.root).write_htmlstring(pretty_print=True)
+    print h.html(presentation.render(h.head, None, None, None), h.root).write_htmlstring(pretty_print=True)
