@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, Net-ng.
+# Copyright (c) 2008, 2009 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -18,12 +18,11 @@ import peak.rules
 
 from lxml import etree as ET
 
-from nagare import var
 from nagare.namespaces import common
 
-CHECK_ATTRIBUTS = False
+CHECK_ATTRIBUTES = False
 
-# Namespace for the ``meld:id`` attribute 
+# Namespace for the ``meld:id`` attribute
 _MELD_NS = 'http://www.plope.com/software/meld3'
 _MELD_ID = '{%s}id' % _MELD_NS
 
@@ -34,7 +33,7 @@ class _Tag(ET.ElementBase):
     """
     def init(self, renderer):
         """Each tag keeps track of the renderer that created it
-        
+
         Return:
            - ``self``
         """
@@ -44,7 +43,7 @@ class _Tag(ET.ElementBase):
     @property
     def renderer(self):
         """Return the renderer that created this tag
-        
+
         Return:
           - the renderer
         """
@@ -53,11 +52,11 @@ class _Tag(ET.ElementBase):
 
     def write_xmlstring(self, encoding='utf-8', pipeline=True, **kw):
         """Serialize in XML the tree beginning at this tag
-        
+
         In:
           - ``encoding`` -- encoding of the XML
-          - ``pipeline`` -- if False, the ``meld:id`` attributs are deleted
-          
+          - ``pipeline`` -- if False, the ``meld:id`` attributes are deleted
+
         Return:
           - the XML
         """
@@ -67,13 +66,24 @@ class _Tag(ET.ElementBase):
 
         return ET.tostring(self, encoding=encoding, method='xml', **kw)
 
+    def xpath(self, *args, **kw):
+        """Override ``xpath()`` to associate a renderer to all the returned nodes
+        """
+        nodes = super(_Tag, self).xpath(*args, **kw)
+
+        renderer = self.renderer
+        for node in nodes:
+            node._renderer = renderer
+
+        return nodes
+
     def findmeld(self, id, default=None):
         """Find a tag with a given ``meld:id`` value
-        
+
         In:
           - ``id`` -- value of the ``meld:id`` attribute to search
           - ``default`` -- value returned if the tag is not found
-          
+
         Return:
           - the tag found, else the ``default`` value
         """
@@ -87,7 +97,7 @@ class _Tag(ET.ElementBase):
 
     def append_text(self, text):
         """Append a text to this tag
-        
+
         In:
           - ``text`` -- text to add
         """
@@ -98,7 +108,7 @@ class _Tag(ET.ElementBase):
 
     def add_child(self, child):
         """Append a child to this tag
-        
+
         In:
           - ``child`` -- child to add
         """
@@ -106,11 +116,11 @@ class _Tag(ET.ElementBase):
         add_child(self, child)
 
     def meld_id(self, id):
-        """Set the value of the attribut ``meld:id`` of this tag
-        
+        """Set the value of the attribute ``meld:id`` of this tag
+
         In:
-          - ``id`` - value of the ``meld;id`` attribut
-          
+          - ``id`` - value of the ``meld;id`` attribute
+
         Return:
           - ``self``
         """
@@ -119,11 +129,11 @@ class _Tag(ET.ElementBase):
 
     def fill(self, *children, **attrib):
         """Change all the child and append attributes of this tag
-        
+
         In:
           - ``children`` -- list of the new children of this tag
           - ``attrib`` -- dictionnary of attributes of this tag
-          
+
         Return:
           - ``self``
         """
@@ -134,31 +144,31 @@ class _Tag(ET.ElementBase):
 
     def replace(self, *children):
         """Replace this tag by others
-        
+
         In:
-          - ``children`` -- list of tags to replace this tag          
+          - ``children`` -- list of tags to replace this tag
         """
         parent = self.getparent()
-        
+
         # We can not replace the root of the tree
         if parent is not None:
             tail = self.tail
             l = len(parent)
-            
+
             i = parent.index(self)
             parent.fill(parent.text or '', parent[:i], children, self.tail or '', parent[i+1:])
 
             if len(parent) >= l:
                 parent[i-l].tail = tail
-    
+
     def repeat(self, iterable, childname=None):
         """Iterate over a sequence, cloning a new child each time
-        
+
         In:
           - ``iterable`` -- the sequence
           - ``childname`` -- If ``None``, clone this tag each time else
             clone each time the tag that have ``childname`` as ``meld:id`` value
-            
+
         Return:
           - list of tuples (cloned child, item of the sequence)
         """
@@ -179,11 +189,11 @@ class _Tag(ET.ElementBase):
 
     def __call__(self, *children, **attrib):
         """Append child and attributes to this tag
-        
+
         In:
           - ``children`` -- children to add
           - ``attrib`` -- attributes to add
-          
+
         Return:
           - ``self``
         """
@@ -196,8 +206,8 @@ class _Tag(ET.ElementBase):
                 children = children[0]
             self.add_child(children)
 
-        if CHECK_ATTRIBUTS and not frozenset(self.attrib).issubset(self._authorized_attribs):
-            raise AttributeError, ("Bad attributes for element <%s>: " % self.tag) + ', '.join(list(frozenset(self.attrib) - self._authorized_attribs))
+        if CHECK_ATTRIBUTES and not frozenset(self.attrib).issubset(self._authorized_attribs):
+            raise AttributeError("Bad attributes for element <%s>: " % self.tag) + ', '.join(list(frozenset(self.attrib) - self._authorized_attribs))
 
         return self
 
@@ -215,23 +225,23 @@ class _Tag(ET.ElementBase):
 
 def add_child(self, o):
     """Default method to add an object to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``o`` -- object to add
-      
+
     Try to add the result of ``o.render()`` to the tag
     """
     render = getattr(o, 'render', None)
     if render is None:
-        raise Exception("Can't append a '%s' to element <%s>" % (type(o), self.tag))
+        raise TypeError("Can't append a '%s' to element <%s>" % (type(o), self.tag))
 
     self.add_child(render(self.renderer))
 
 @peak.rules.when(add_child, (_Tag, basestring))
 def add_child(self, s):
     """Add a string to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``s`` - str or unicode string to add
@@ -241,11 +251,11 @@ def add_child(self, s):
 @peak.rules.when(add_child, (_Tag, tuple))
 def add_child(self, t):
     """Add a tuple to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``t`` -- tuple to add
-      
+
     Add each items of the tuple
     """
     for child in t:
@@ -254,11 +264,11 @@ def add_child(self, t):
 @peak.rules.when(add_child, (_Tag, list))
 def add_child(self, l):
     """Add a list to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``l`` -- list to add
-      
+
     Add each items of the list
     """
     for child in l:
@@ -267,11 +277,11 @@ def add_child(self, l):
 @peak.rules.when(add_child, (_Tag, types.GeneratorType))
 def add_child(self, g):
     """Add a generator to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``l`` -- generator to add
-      
+
     Add each items produced
     """
     for child in g:
@@ -280,31 +290,43 @@ def add_child(self, g):
 @peak.rules.when(add_child, (_Tag, int))
 def add_child(self, i):
     """Add an integer to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``i`` -- integer to add
-      
-    Convert the integer to string and then add it 
+
+    Convert the integer to string and then add it
     """
     self.append_text(str(i))
 
+@peak.rules.when(add_child, (_Tag, long))
+def add_child(self, l):
+    """Add a long integer to a tag
+
+    In:
+      - ``self`` -- the tag
+      - ``l`` -- long integer to add
+
+    Convert the long integer to string and then add it
+    """
+    self.append_text(str(l))
+
 @peak.rules.when(add_child, (_Tag, float))
 def add_child(self, f):
     """Add a float to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``f`` -- float to add
-      
-    Convert the float to string and then add it 
+
+    Convert the float to string and then add it
     """
     self.append_text(str(f))
 
 @peak.rules.when(add_child, (_Tag, ET.ElementBase))
 def add_child(self, element):
     """Add a tag to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``element`` -- the tag to add
@@ -317,7 +339,7 @@ def add_child(self, element):
 @peak.rules.when(add_child, (_Tag, ET._Comment))
 def add_child(self, element):
     """Add a comment element to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``element`` -- the comment to add
@@ -330,11 +352,11 @@ def add_child(self, element):
 @peak.rules.when(add_child, (_Tag, ET._ProcessingInstruction))
 def add_child(self, element):
     """Add a PI element to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``element`` -- the PI to add
-      
+
     Do nothing
     """
     pass
@@ -342,74 +364,74 @@ def add_child(self, element):
 @peak.rules.when(add_child, (_Tag, dict))
 def add_child(self, d):
     """Add a dictionary to a tag
-    
+
     In:
       - ``self`` -- the tag
       - ``element`` -- the dictionary to add
-      
+
     Each key/value becomes an attribute of the tag
-    
+
     Attribute name can end with a '_' which is removed
     """
     for (name, value) in d.items():
         if name.endswith('_'):
             name = name[:-1]
 
-        add_attribut(self, name, value)
+        add_attribute(self, name, value)
 
 # ---------------------------------------------------------------------------
 
-# Generic methods to add an attribut to a tag
-# -------------------------------------------
+# Generic methods to add an attribute to a tag
+# --------------------------------------------
+
+def add_attribute(self, name, value):
+    """Default method to add an attribute to a tag
 
-def add_attribut(self, name, value):
-    """Default method to add an attribut to a tag
-    
     In:
       - ``self`` -- the tag
-      - ``name`` -- name of the attribut to add
-      - ``value`` -- value of the attribut to add
+      - ``name`` -- name of the attribute to add
+      - ``value`` -- value of the attribute to add
     """
-    add_attribut(self, name, unicode(value))
+    add_attribute(self, name, unicode(value))
 
-@peak.rules.when(add_attribut, (_Tag, basestring, basestring))
-def add_attribut(self, name, value):
+@peak.rules.when(add_attribute, (_Tag, basestring, basestring))
+def add_attribute(self, name, value):
     self.set(name, value)
 
 # ---------------------------------------------------------------------------
 
 class TagProp(object):
-    """Tag factory with a behaviour of an object attribut
-    
-    Each time this attribut is read, a new tag is created
+    """Tag factory with a behavior of an object attribute
+
+    Each time this attribute is read, a new tag is created
     """
     def __init__(self, name, authorized_attribs, factory=None):
-        """Initialisation
-        
+        """Initialization
+
         In:
           - ``name`` -- name of the tags to create
-          - ``authorized_attribs`` -- names of the valid attributs
+          - ``authorized_attribs`` -- names of the valid attributes
           - ``factory`` -- special factory to create the tag
         """
         self._name = name
         self._factory = factory
 
-        if CHECK_ATTRIBUTS:
+        if CHECK_ATTRIBUTES:
             self._authorized_attribs = frozenset(authorized_attribs)
 
     def __get__(self, renderer, cls):
-        """Create a new tag each time this attribut is read
-        
+        """Create a new tag each time this attribute is read
+
         In:
-          - ``renderer`` -- the object that has this attribut
+          - ``renderer`` -- the object that has this attribute
           - ``cls`` -- *not used*
-          
+
         Return:
           - a new tag
         """
         element = renderer.makeelement(self._name)
 
-        if CHECK_ATTRIBUTS:
+        if CHECK_ATTRIBUTES:
             element._authorized_attribs = self._authorized_attribs
 
         return element
@@ -417,25 +439,25 @@ class TagProp(object):
 
 class RendererMetaClass(type):
     """Meta class for the renderer class
-    
+
     Discover the tags that have a special factory and pass them to the
     ``class_init()`` method of the class
     """
     def __new__(self, cls_name, bases, ns):
         """Creation of the class
-        
+
         In:
           - ``cls_name`` -- name of the class to create
           - ``bases`` -- tuple of the base classes of the class to create
           - ``ns`` -- namespace of the class to create
-          
+
         Return:
           - the new class
         """
         cls = super(RendererMetaClass, self).__new__(self, cls_name, bases, ns)
 
         special_tags = {}
-        
+
         for (k, v) in ns.items():
             if isinstance(v, TagProp):
                 if v._factory is not None:
@@ -448,14 +470,6 @@ class RendererMetaClass(type):
 
 # -----------------------------------------------------------------------
 
-class DummyRenderer(object):
-    """Root of the ``XmlRenderer`` objects
-    """
-    def __init__(self):
-        self.namespaces = None
-        self._default_namespace = None
-
-
 class XmlRenderer(common.Renderer):
     """The base class of all the renderers that generate a XML dialect
     """
@@ -464,7 +478,7 @@ class XmlRenderer(common.Renderer):
     @classmethod
     def class_init(cls, special_tags):
         """Class initialisation
-        
+
         In:
           -- ``special_tags`` -- tags that have a special factory
         """
@@ -474,45 +488,46 @@ class XmlRenderer(common.Renderer):
 
     def __init__(self, parent=None):
         """Renderer initialisation
-        
+
         In:
           - ``parent`` -- parent renderer
         """
         if parent is None:
-            parent = DummyRenderer()
+            self.namespaces = None
+            self._default_namespace = None
+        else:
+            # This renderer use the same XML namespaces than its parent
+            self.namespaces = parent.namespaces
+            self._default_namespace = parent._default_namespace
 
         self.parent = parent
-
-        # This renderer use the same XML namespaces than its parent
-        self.namespaces = parent.namespaces
-        self._default_namespace = parent._default_namespace
         self._prefix = ''
 
         # The stack, contening the last tag push by a ``with`` statement
         # Initialized with a dummy root
         self._stack = [self.makeelement('_renderer_root_')]
-        
+
         # Each renderer created has a unique id
         self.id = self.generate_id('id')
 
     def _get_default_namespace(self):
         """Return the default_namespace
-        
+
         Return:
           - the default namespace or ``None`` if no default namespace was set
         """
         return self._default_namespace
-    
+
     def _set_default_namespace(self, namespace):
         """Change the default namespace
 
         The namespace must be a key of the ``self.namespaces`` dictionary or be
         ``None``
-        
+
         For example:
-        
+
         .. code-block:: python
-        
+
           x.namespaces = { 'xsl' : 'http://www.w3.org/1999/XSL/Transform' }
           x.set_default_namespace('xsl')
         """
@@ -527,30 +542,30 @@ class XmlRenderer(common.Renderer):
     @property
     def root(self):
         """Return the first tag(s) sent to the renderer
-        
+
         .. warning::
             A list of tags can be returned
-        
+
         Return:
           - the tag(s)
         """
         children = self._stack[0].getchildren()
-        
+
         text = self._stack[0].text
         if text is not None:
             children.insert(0, text)
-        
+
         if len(children) == 1:
             children = children[0]
-        
+
         return children
 
     def enter(self, current):
         """A new tag is pushed by a ``with`` statement
-        
+
         In:
           - ``current`` -- the tag
-          
+
         Return:
           - ``current``
         """
@@ -566,11 +581,11 @@ class XmlRenderer(common.Renderer):
 
     def _makeelement(self, tag, parser):
         """Make a tag, in the default namespace set
-        
+
         In:
           - ``tag`` -- name of the tag to create
           - ``parser`` -- parser used to create the tag
-          
+
         Return:
           - the new tag
         """
@@ -581,10 +596,10 @@ class XmlRenderer(common.Renderer):
 
     def makeelement(self, tag):
         """Make a tag, in the default namespace set
-        
+
         In:
           - ``tag`` -- name of the tag to create
-          
+
         Return:
           - the new tag
         """
@@ -593,10 +608,10 @@ class XmlRenderer(common.Renderer):
 
     def __lshift__(self, current):
         """Add a tag tag the last tag pushed by a ``with`` statement
-        
+
         In:
           - ``current`` -- tag to add
-        
+
         Return:
           - ``self``, the renderer
         """
@@ -605,10 +620,10 @@ class XmlRenderer(common.Renderer):
 
     def comment(self, text=''):
         """Create a comment element
-        
+
         In:
           - ``text`` -- text of the comment
-          
+
         Return:
           - the new comment element
         """
@@ -616,7 +631,7 @@ class XmlRenderer(common.Renderer):
 
     def parse_xml(self, source, fragment=False, no_leading_text=False, **kw):
         """Parse a XML file
-        
+
         In:
           - ``source`` -- can be a filename or a file object
           - ``fragment`` -- if ``True``, can parse a XML fragment i.e a XML without
@@ -624,20 +639,20 @@ class XmlRenderer(common.Renderer):
           - ``no_leading_text`` -- if ``fragment`` is ``True``, ``no_leading_text``
             is ``False`` and the XML to parsed begins by a text, this text is keeped
           - ``kw`` -- keywords parameters are passed to the XML parser
-          
+
         Return:
           - the root element of the parsed XML, if ``fragment`` is ``False``
-          - a list of XML elements, if ``fragment`` is ``True`` 
+          - a list of XML elements, if ``fragment`` is ``True``
         """
         # Create a dedicated XML parser with the ``kw`` parameter
         parser = ET.XMLParser(**kw)
         # This parser will generate nodes of type ``_Tag``
         parser.setElementClassLookup(ET.ElementDefaultClassLookup(element=_Tag))
-        
+
         if not fragment:
             # Parse a XML file
             # ----------------
-            
+
             root = ET.parse(source, parser).getroot()
             # Attach the renderer to the root
             root._renderer = self
@@ -645,21 +660,21 @@ class XmlRenderer(common.Renderer):
 
         # Parse a XML fragment
         # --------------------
-        
+
         # Create a dummy root
         source = cStringIO.StringIO('<dummy>%s</dummy>' % source.read())
         root = ET.parse(source, parser).getroot()
-        
+
         for e in root[:]:
             # Attach the renderer to each roots
             e._renderer = self
-        
-        # Return the children of the dummy root    
+
+        # Return the children of the dummy root
         return ([root.text] if root.text and not no_leading_text else []) + root[:]
 
     def parse_xmlstring(self, text, fragment=False, no_leading_text=False, **kw):
         """Parse a XML string
-        
+
         In:
           - ``text`` -- can be a ``str`` or ``unicode`` string
           - ``fragment`` -- if ``True``, can parse a XML fragment i.e a XML without
@@ -667,35 +682,35 @@ class XmlRenderer(common.Renderer):
           - ``no_leading_text`` -- if ``fragment`` is ``True``, ``no_leading_text``
             is ``False`` and the XML to parsed begins by a text, this text is keeped
           - ``kw`` -- keywords parameters are passed to the XML parser
-          
+
         Return:
           - the root element of the parsed XML, if ``fragment`` is ``False``
-          - a list of XML elements, if ``fragment`` is ``True`` 
+          - a list of XML elements, if ``fragment`` is ``True``
         """
         if type(text) == unicode:
             text = text.encode(kw.setdefault('encoding', 'utf-8'))
-        
+
         return self.parse_xml(cStringIO.StringIO(text), fragment, no_leading_text, **kw)
 
 # ---------------------------------------------------------------------------
 
 class Renderer(XmlRenderer):
     """The XML Renderer
-    
+
     This renderer generate any tags you give
-    
+
     .. code-block:: pycon
-    
+
        >>> xml = xml.Renderer()
        >>> xml.foo(xml.bar).write_xmlstring()
        '<foo><bar/></foo>'
     """
     def __getattr__(self, name):
         """Any attribute access becomes a tag generation
-        
+
         In:
           - ``name`` -- name of the tag to generate
-          
+
         Return:
           - the generated tag
         """
@@ -715,14 +730,14 @@ if __name__ == '__main__':
 
     print x.root.write_xmlstring(xml_declaration=True, pretty_print=True)
     print
-    
+
     for (e, (name, addr)) in x.root.repeat((('bill', 'seatle'), ('steve', 'cupertino')), 'contact'):
          e.findmeld('name').text = name
          e.findmeld('addr').text = addr
-    
+
     print x.root.write_xmlstring(pretty_print=True)
     print
-    
+
     # -----------------------------------------------------------------------
 
     x = Renderer()
@@ -731,19 +746,19 @@ if __name__ == '__main__':
 
     print x.all(x.root).write_xmlstring(xml_declaration=True, pretty_print=True)
     print
-    
+
     # -----------------------------------------------------------------------
-    
+
     t1 = x.parse_xmlstring('''
     <a>avant<x>kjkjkl</x>#<b b='b'>hello</b>apres</a>
     ''')
-    
+
     t2 = x.parse_xmlstring('''
     <toto>xxx<titi a='a'>world</titi>yyy</toto>
     ''')
-    
+
     t1[1].replace(t2[0])
     #t1[1].replace('new')
     #t1[1].replace()
-    
+
     print t1.write_xmlstring()
