@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, Net-ng.
+# Copyright (c) 2008, 2009 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -9,6 +9,7 @@
 
 """The HTTP publisher"""
 
+import time
 from paste import httpserver
 
 from nagare.publishers import common
@@ -36,4 +37,5 @@ class Publisher(common.Publisher):
         
         # The puslisher is a threaded server so call once the ``on_new_process()`` method
         self.on_new_process()
+        print time.strftime('%x %X -', time.localtime()),
         httpserver.serve(self.urls, host, port, **conf)
