@@ -1,5 +1,5 @@
 #--
-# Copyright (c) 2008, Net-ng.
+# Copyright (c) 2008, 2009 Net-ng.
 # All rights reserved.
 #
 # This software is licensed under the BSD License, as described in
@@ -11,8 +11,8 @@
  
 from nagare.security import common
 
-class Manager(common.Authentification, common.Rules):
-    """A security manager is typically a mix-in of an authentification
+class Manager(common.Authentication, common.Rules):
+    """A security manager is typically a mix-in of an authentication
     manager and security rules
     """
     pass
